#! /usr/bin/perl -X
#****s* SecDec/loop/decompose.pl
#
#  NAME
#  decompose.pl
#
#  USAGE
#  ./decompose.pl or ./launch
# 
#  USES 
#  $paramfile, header.pm, mathlaunch.pl
#
#  USED BY 
#  launch 
#      
#  PURPOSE
#  doing the iterated sector decomposition in Mathematica
#  uses a different decomposition strategy if infinite recursion occurs
#  the primary sectors for which a different strategy should be used 
#  must be given at the end of the $paramfile
#    
#  INPUTS
#  $paramfile (default is param.input), read via module header
#  $templatefile (default is Template.m)
#    
#  RESULT    
#  -adds decomposition part to the file $currentdir/$graph.m 
#  -launches the decomposition and writes the results to 
#   $currentdir in the form of lists containing the 
#   parametric functions for each pole structure
#  OPTIONS
#  to use a param.input file with a different name
#  use option "-p paramfile" 
#  to use a template file with the a different name
#  use option "-t templatefile"
#  to specify a different directory to work in
#  use option "-d workingdirectory" 
# 
#****
##################################
use Getopt::Long;
GetOptions("parameter=s" => \$paramfile, "template=s"=>\$templatefile, "dirwork=s"=>\$workingdir);
unless ($paramfile) {
  $paramfile = "param.input";
}
unless ($templatefile) {
  $templatefile = "Template.m";
}
$wparamfile=$paramfile;
if($workingdir){
 $workingdir=~s/\/$//;
 $wparamfile="$workingdir/$paramfile";
 $templatefile="$workingdir/$templatefile";
 $wdstring=" -d $workingdir";
}
use lib "perlsrc";
use header;
use getinfo;
use dirform;
system("perl -pi -e 's/\\/(\\s*)\\n/\\n/g' $wparamfile");
my %hash_var=header::readparams($wparamfile);
$dirbase=`pwd`;
chomp $dirbase;
$subdir=$hash_var{"diry"};
$diry=dirform::norm("${dirbase}/$subdir");
$currentdir=$hash_var{"currentdir"};
$graph=$hash_var{"graph"};
$exe=$hash_var{"exe"};
if($exe ne "0"){unless($exe){$exe=4}};
$clusterflag=$hash_var{"clusterflag"};
unless($clusterflag){$clusterflag=0};
unless($graph){die "must give the graph name in $paramfile\n"};
$together=$hash_var{"together"};
$fromlaunch=$ARGV[0];
unless($fromlaunch){$fromlaunch=""};
if ($fromlaunch ne "launch") {
print "graph = $graph\n";
}
$externalegs=$hash_var{"externalegs"};
unless ($currentdir) {
if($workingdir){
if ($workingdir=~m/^\//){
   $diry=dirform::norm("$workingdir/$subdir");
  $currentdir="$diry/$graph";
 } else {
  $subdir="$workingdir/$subdir";
  $diry=dirform::norm("${dirbase}/$subdir");
  $currentdir="$diry/$graph"
 }
} else {
 $currentdir=dirform::norm("$diry/$graph")
}
}
$Nn=$hash_var{"Nn"};
$Nmin=1;
$Nmax=$Nn;
@Nlist = split(/,/,$hash_var{"Nlist"});
$indflag=0;
if (@Nlist){
 $indflag=1
}else{
 print "results of the decomposition will be written to\n";
 print "$currentdir\n";
}
unless (@Nlist) {
@Nlist=($Nmin .. $Nmax)
};
$Nmin=$Nlist[0];
$last=eval(@Nlist-1);
$Nmax=$Nlist[$last];
@multiplicities = split(/,/,$hash_var{"multlist"});
unless (@multiplicities) {
@multiplicities = (1,@Nlist);
}
$el=0;
foreach $sec (@Nlist) {
if (@multiplicities) {
$hashmulti{ $sec } = $multiplicities[$el];
}
else{
$hashmulti{ $sec } = 1;
}
$el++;
}
@nonstoplist = split(/,/,$hash_var{"nonstop"});
$nbnonstop=@nonstoplist;
#
############## end of user defined input ############
#
# insert main path in Template.m
$itermax=1;
if ($indflag==1){
 $itermax=@Nlist;
 open (ALLSEC,">","decompose$graph");
 print ALLSEC "echo \"results of the decomposition will be written to\"\n";
 print ALLSEC "echo \"$currentdir\"\n";
};
for ($iti=0;$iti<$itermax;$iti++){
 if($indflag==1){$Nmin=$Nlist[$iti];$Nmax=$Nmin};
 open (EREAD,"$templatefile") || die "cannot open $templatefile\n";
 open (ET,">tmp$graph") || die "cannot open tmp$graph\n";
 while(<EREAD>) {
  chomp;
print ET "$_\n";
}
close (EREAD);
$templatetail="$dirbase/src/deco/template_tail.m";
open (EREAD,"$templatetail") || die "cannot open $templatetail\n";
while(<EREAD>) {
  chomp;
  s/path\s*=\s*(.*)/path=\"$dirbase\/\"/;
  s/graphstring\s*=\s*ToString\[\w.*\]\s*/graphstring="$graph"/;
  s/externalegs\s*=\s*\d*/externalegs=$externalegs/;
  s/currentdir\s*=\s*(.*)/currentdir=\"$currentdir\/\"/;
  s/npmin\s*=\s*(\d.*)/npmin=${Nmin};/;
  s/npmax\s*=\s*(.*)/npmax=${Nmax};/;
  print ET "$_\n";
 }
 close (EREAD);
 print ET "\n";
 print ET "Get[StringJoin[currentdir,ToString[\"FUN.m\"]]];\n";
 print ET "\n";
 print ET "If[MatchQ[F[z],0],\n";
 print ET "Print[\"Warning, F[z]=0, please verify your input\"];\n";
 print ET ",\n";
 print ET "signcheck={z[a_]->1/2,ssp[b_]->-1,sp[c__]->-1,ms[d_]->1};\n";
 if ($hash_var{"contourdef"} eq "True"){
  print ET "If[False,\n";
 }else{
  print ET "If[(F[z]/.signcheck)<0,\n";
 }
 print ET "Print[\"Warning, F[z] is not positive semi-definite, please verify your input\"];\n";
 print ET ",\n";
 print ET "(* prepare the input for decomposition *)\n";
 print ET "\n";
 print ET "Get[StringJoin[path,ToString[\"src/deco/prepareinput.m\"]]];\n";
 print ET "\n";
 print ET "(* do the iterated decomposition *)\n";
 print ET "\n";
 $qlist=join(",",@nonstoplist);
 print ET "qlist=\{$qlist\};\n";
 print ET "Get[StringJoin[path,ToString[\"src/deco/decomposition.m\"]]];\n";
 print ET "\n";
 print ET "(* prepare the output files *)\n";
 print ET "\n";
# print ET "Get[StringJoin[path,ToString[\"src/deco/makeoutput.m\"]]];\n";
 print ET "]];\n";
 close (ET);

#####################################################
 chdir("$dirbase") || die "cannot change to directory $dirbase\n";
 $logex="";
 if($Nmin==$Nmax){$logex="Sec$Nmin"}; 
 system("mv $dirbase/tmp${graph} $currentdir/${graph}$logex.m");
 if($indflag==0){
  print "doing sector decomposition . . . \n";
  system ("perl perlsrc/mathlaunch.pl $currentdir/${graph}$logex.m $currentdir/${graph}Decomposition$logex.log");
  $wflag=getinfo::decwarning("$currentdir/${graph}Decomposition$logex.log");
  if($wflag==0){
   print "done\n";
  } else {
   print "Error in F[z]. Please check your input, in particular the on-shell conditions\n";
   exit 10;
  }
 } else {
  open (SECLAUNCH,">","$currentdir/launch${graph}$logex");
  print SECLAUNCH "cd $dirbase\n";
  print SECLAUNCH "if [ ! \$1 ]\nthen\n";
  print SECLAUNCH "echo \"results of the decomposition will be written to\"\n";
  print SECLAUNCH "echo \"$currentdir\"\n";
  print SECLAUNCH "echo \"Decomposing $logex\"\nfi\n";
  print SECLAUNCH "perl perlsrc/mathlaunch.pl $currentdir/${graph}$logex.m $currentdir/${graph}Decomposition$logex.log";
  close (SECLAUNCH);
  system("chmod +x $currentdir/launch${graph}$logex");
  print ALLSEC "echo \"Decomposing $logex\"\n";
  print ALLSEC "$currentdir/launch${graph}$logex 1\n";
 }
} # end for ($iti=0;$iti<$itermax;$iti++)
if ($indflag==1){
 print ALLSEC "cd $dirbase\n";
 print ALLSEC "./subexp.pl -p $paramfile$wdstring\n";
 print ALLSEC "echo\necho\n";
 if($exe-$clusterflag==4){
  print ALLSEC "./results.pl -p $paramfile$wdstring\n"
 }elsif($exe==4){
  print ALLSEC "echo \"When all jobs are completed, cd to \"\n";
  print ALLSEC "echo \"$dirbase\"\n";
  print ALLSEC "echo \"and run results.pl -p $paramfile$wdstring\"\n"
 }else{
  if($clusterflag==0){
   print ALLSEC "echo \"to complete the calculation, cd to\"\n";
   print ALLSEC "echo \"$dirbase\"\n";
   print ALLSEC "echo \"and run finishnumerics.pl -p $paramfile$wdstring\"\n"
  }elsif($exe==0){
   print ALLSEC "echo \"to complete the calculation, cd to\"\n";
   print ALLSEC "echo \"$dirbase\"\n";
   print ALLSEC "echo \"and run finishnumerics.pl -p $paramfile$wdstring\"\n"
  }else{
   if($together==0){
    print ALLSEC "echo \"to complete the calculation, wait for all jobs to finish, then cd to\"\n";
    print ALLSEC "echo \"$dirbase\"\n";
    print ALLSEC "echo \"and run finishnumerics.pl -p $paramfile$wdstring\"\n"
   }
  }
 }
   
 ;
 close ALLSEC;
 system("chmod +x decompose$graph");
 print "To run the decomposition in series, type '$dirbase/decompose$graph'\n";
 print "To distribute decomposition over a number of cores, run the executables\n";
 print "$currentdir/launch${graph}Sec*\n";
 print "after the decomposition is done, cd to $dirbase and type\n './subexp.pl -p $paramfile$wdstring'\n";
 print "(This is done automatically when run in series)\n";
 print "to collect the results, type './results.pl -p $paramfile$wdstring'\n";
} 
print "\n"; 
#####################################################
