  #****p* SecDec/loop/perlsrc/makeintC.pm
  #  NAME
  #    makeintC.pm
  #
  #  USAGE
  #  is called from preparenumerics.pl via writefiles::makeintC
  # 
  #  USES 
  #
  #  $paramfile, header.pm, arguments parsed from preparenumerics.pl
  #
  #  USED BY 
  #    
  #  preparenumerics.pl, writefiles.pm
  #
  #  PURPOSE
  #  writes the fortran files *intfile*.c in the appropriate subdirectory
  #    
  #  INPUTS
  #  
  #  arguments:
  #  filename:name and directory of the *intfile*.c to be written
  #  lowii: lowest individual function number to integrate
  #  highii: highest individual function number to integrate
  #  prestring: prefix of the functions to be integrated
  #  jj:epsilon order
  #  kk:which integration file is to be written, specifies *intfile$kk.c
  #  numvar: number of variables in integration
  #  maxpole: maximal pole for this pole structure (can be spurious)
  #  paramfile: file to read parameters from
  #
  #  parameters read from $paramfile:
  #  graph:name of graph
  #  point:name of numerical point being calculated
  #  @stupoint:array of mandelstam variables
  #  @psq:array of values of external momenta^2
  #  @msq:array of propagator masses
  #  @ncall:number of calls BASES uses
  #  @iter1:number of interations BASES uses for its first run
  #  @iter2:number of interations BASES uses for its second run
  #  @acc1:accuracy BASES aims for on its first run
  #  @acc2:accuracy BASES aims for on its second run
  #  cuba parameters - see Cuba-2.1/Cuba.pdf
  #  externalegs:number of external legs for the process
  #
  #  
  #    
  #  RESULT
  #  new functions $[point]intfile$[kk].c in the appropriate subdirectory graph/polestructure/epsilonorder/
  #    
  #  SEE ALSO
  #  preparenumerics.pl
  #   
  #****
use lib "perlsrc";
use header;
package makeintC;

sub go {
 my $filename=$_[0];
 my $lowii=$_[1];
 my $highii=$_[2];
 my $prestring=$_[3];
 my $jj=$_[4];
 my $kk=$_[5];
 my $numvar=$_[6];
 my $maxpole=$_[7];
 my $paramfile=$_[8];
 my %hash_var=header::readparams($paramfile);
 my $graph=$hash_var{"graph"};
 my $point=$hash_var{"pointname"};
 my @stupointM = split(/,/,$hash_var{"stupoint"});
 my @psqM = split(/,/,$hash_var{"psq"});
 my @msqM = split(/,/,$hash_var{"msq"});
 my $k = $jj - $maxpole;
 my $routine=$hash_var{"integrator"};
 my $cubapath=$hash_var{"cubapath"};
# my $compiler=$hash_var{"compiler"};
# unless($compiler){$compiler="g++"};
 $dirbase=`pwd`;
 chomp $dirbase;
 unless ($cubapath) {$cubapath="$dirbase/Cuba-2.1"};
 $cubapath=~s/\/loop\//\//;
# $cubapath="$cubapath/$compiler";
 my $seed=$hash_var{"seed"};
 unless($seed){$seed=0};
 unless($routine){$routine=1};#vegas
 @epsrel=split(/,/,$hash_var{"epsrel"});
 dtoe(@epsrel);
 unless($epsrel[$k]){$epsrel[$k]="1e-3"};
 @epsabs=split(/,/,$hash_var{"epsabs"});
 dtoe(@epsabs);
 unless($epsabs[$k]){$epsabs[$k]="1e-12"};
 $flags=$hash_var{"cubaflags"};
 if ($flags ne "0"){unless($flags){$flags=2}};
 @mineval=split(/,/,$hash_var{"mineval"});
 unless ($mineval[$k]){$mineval[$k]=0};
 @maxeval=split(/,/,$hash_var{"maxeval"});
 unless ($maxeval[$k]){$maxeval[$k]=100000000};
if ($routine==1){#vegas
  @nstart=split(/,/,$hash_var{"nstart"});
  unless($nstart[$k]){$nstart[$k]=1000000};
  @nincrease=split(/,/,$hash_var{"nincrease"});
  unless($nincrease[$k]){$nincrease[$k]=500000};
 }elsif($routine==2){#suave
  @nnew=split(/,/,$hash_var{"nnew"});
  unless($nnew[$k]){$nnew[$k]=1000000};
  @flatness=split(/,/,$hash_var{"nnew"});
  dtoe(@flatness);
  unless($flatness[$k]){$flatness[$k]="1"};
 }elsif($routine==3){#divonne
  @key1=split(/,/,$hash_var{"key1"});
  unless($key1[$k]){$key1[$k]=1000};
  @key2=split(/,/,$hash_var{"key2"});
  unless($key2[$k]){$key2[$k]=1};
  @key3=split(/,/,$hash_var{"key3"});
  unless($key3[$k]){$key3[$k]=1};
  @maxpass=split(/,/,$hash_var{"maxpass"});
  unless($maxpass[$k]){$maxpass[$k]=5};
  @border=split(/,/,$hash_var{"border"});
  dtoe(@border);
  if($border[$k] ne "0"){unless($border[$k]){$border[$k]="1e-8"}};
  @maxchisq=split(/,/,$hash_var{"maxchisq"});
  dtoe(@maxchisq);
  unless($maxchisq[$k]){$maxchisq[$k]="10d0"};
  @mindeviation=split(/,/,$hash_var{"mindeviation"});
  dtoe(@mindeviation);
  unless($mindeviation[$k]){$mindeviation[$k]=".25"};
  @nextra=split(/,/,$hash_var{"nextra"});
  unless($nextra[$k]){$nextra[$k]=0};
 }elsif($routine==4){#cuhre
  @key=split(/,/,$hash_var{"key"});
  unless($key[$k]){$key[$k]=7};
 }
 my $externalegs=$hash_var{"externalegs"};
 foreach $s (@stupointM) {
  chomp;
  if(substr($s,0,1) eq "-"){
   $sE=substr($s,1)
  }else{
   $sE=eval(-$s)
  };
  $sE="${sE}";
  push(@stupoint,$sE);
 }
 my $eslen=@stupoint;
 if ($eslen){
  $esdefine=join(",",@stupoint);
 }else{
  $eslen=1;$esdefine="0.0"
 }
 $esdefine="double es[$eslen]={$esdefine};\n";
 
 foreach $s (@psqM) {
  chomp;
  if(substr($s,0,1) eq "-"){
   $sE=substr($s,1)
  }else{
   $sE=eval(-$s)
  };
  $sE="${sE}";
  push(@psq,$sE);
 }
 my $psqlen =@psq;
 if ($psqlen){
  $psqdefine=join(",",@psq);
 }else{
  $psqlen=1;$psqdefine="0.0"
 }
 $psqdefine="double esx[$psqlen]={$psqdefine};\n";
 foreach $s (@msqM) {
  $sE=eval($s);
  $sE="${sE}";
  push(@msq,$sE);
 }
 my $msqlen =@msq;
 if ($msqlen){
  $msqdefine=join(",",@msq);
 }else{
  $msqlen=1;$msqdefine="0.0"
 }
 $msqdefine="double em[$msqlen]={$msqdefine};\n";
 $xlambda=$hash_var{"xlambda"};unless($xlambda){$xlambda="0.1"};
 $xalpha=$hash_var{"xalpha"};unless($xalpha){$xalpha="1.0"};
 $xbeta=$hash_var{"xbeta"};unless($xbeta){$xbeta="1.0"};
 my $xvalsdefine="double xvals[3]={$xlambda,$xalpha,$xbeta};\n";
 $functiondeclare="";$functioncall="";
 $declareargs="(const double x[],double es[],double esx[],double em[],double xvals[])";
 $callargs="(xx,es,esx,em,xvals)";
 $numberoffunctions=0;
 for ($ii=$lowii;$ii<=$highii;$ii++){
  $function="${prestring}f$ii";
  $functiondeclare="${functiondeclare}dcmplx $function$declareargs;\n";
  $functioncall="$functioncall f0[$numberoffunctions]=$function$callargs;\n";
  $numberoffunctions++
 }
 @declarations=();
 push(@declarations,"clock_t start, end");
 push(@declarations,"double cpu_time_used");
 push(@declarations,"double ESTIM_RE =0.e0");
 push(@declarations,"double ESTIM_IM =0.e0");
 push(@declarations,"double SIGMA_RE =0.e0");
 push(@declarations,"double SIGMA_IM =0.e0");
 push(@declarations,"double CHI_RE =0.e0");
 push(@declarations,"double CHI_IM =0.e0");
 push(@declarations,"int method = $routine");
 push(@declarations,"const int ndim = $numvar");
 push(@declarations,"const int ncomp=2");
 push(@declarations,"double integral[ncomp]={0.0}");
 push(@declarations,"double error[ncomp]={0.0}");
 push(@declarations,"double prob[ncomp]={0.0}");
 push(@declarations,"const double EPSREL=$epsrel[$k]");
 push(@declarations,"const double EPSABS=$epsabs[$k]");
 push(@declarations,"const int FLAGS=$flags");
 push(@declarations,"const int SEED=$seed");
 push(@declarations,"const int MINEVAL=$mineval[$k]");
 push(@declarations,"const int MAXEVAL=$maxeval[$k]");
 push(@declarations,"int NREGIONS=0");
 push(@declarations,"int NEVAL=0");
 push(@declarations,"int FAIL=0");
 if($routine==1){
  push(@declarations,"const int NSTART=$nstart[$k]");
  push(@declarations,"const int NINCREASE=$nincrease[$k]");
  push(@declarations,"const int NBATCH=1000");
  push(@declarations,"const int GRIDNO=0");
  push(@declarations,"const char *STATEFILE=\"vegas_status.dat\"");
  $integratorcall="Vegas(ndim, ncomp, Integrand, USERDATA,
	    EPSREL, EPSABS, FLAGS, SEED, MINEVAL, MAXEVAL, NSTART, 
	    NINCREASE, NBATCH, GRIDNO, STATEFILE, &NEVAL, &FAIL, 
	    integral, error, prob);\n";
 }elsif($routine==2){
  push(@declarations,"const int NNEW=$nnew[$k]");
  push(@declarations,"const double FLATNESS=$flatness[$k]");
  $integratorcall="Suave(ndim, ncomp, Integrand, USERDATA,
	    EPSREL, EPSABS, FLAGS, SEED,
	    MINEVAL, MAXEVAL,    NNEW, FLATNESS,
	    &NREGIONS, &NEVAL, &FAIL, integral, error, prob);\n";
 }elsif($routine==3){
  push(@declarations,"const int KEY1=$key1[$k]");
  push(@declarations,"const int KEY2=$key2[$k]");
  push(@declarations,"const int KEY3=$key3[$k]");
  push(@declarations,"const int MAXPASS=$maxpass[$k]");
  push(@declarations,"const double BORDER=$border[$k]");
  push(@declarations,"const double MAXCHISQ=$maxchisq[$k]");
  push(@declarations,"const double MINDEVIATION=$mindeviation[$k]");
  push(@declarations,"const int NGIVEN=0");
  push(@declarations,"const int LDXGIVEN=ndim");
  push(@declarations,"double XGIVEN[]={0.0}");
  push(@declarations,"const int NEXTRA=$nextra[$k]");
  $integratorcall="Divonne(ndim, ncomp, Integrand, USERDATA,
	      EPSREL, EPSABS, FLAGS, SEED,
	      MINEVAL, MAXEVAL, KEY1, KEY2, KEY3, MAXPASS,
	      BORDER, MAXCHISQ, MINDEVIATION,
	      NGIVEN, LDXGIVEN, XGIVEN, NEXTRA, NULL,
	      &NREGIONS, &NEVAL, &FAIL, integral, error, prob);\n";
 }elsif($routine==4){
  push(@declarations,"const int KEY=$key[$k]");
  $integratorcall="Cuhre(ndim, ncomp, Integrand, USERDATA,
	    EPSREL, EPSABS, FLAGS,
	    MINEVAL, MAXEVAL, KEY,
	    &NREGIONS, &NEVAL, &FAIL, integral, error, prob);\n";
 }
 $decs=join(";\n ",@declarations);
 @headers=("\"$cubapath/cuba.h\"","<math.h>","<complex>","<fstream>","<time.h>","<iostream>");
 $headstring=join("\n#include ",@headers);
 $headstring="#include $headstring";
 if(-e $filename){system("rm -f $filename")};
 open(INTFILE, ">", "$filename");
  print INTFILE "$headstring\n";
  print INTFILE "using namespace std;\n";
  print INTFILE "typedef complex<double> dcmplx;\n";
  print INTFILE "$functiondeclare";
  print INTFILE "int Integrand (const int *ndim, const double xx[], const int *ncomp, double ff[], void *userdata);\n";
  print INTFILE "$esdefine";
  print INTFILE "$psqdefine";
  print INTFILE "$msqdefine";
  print INTFILE "$xvalsdefine";
  print INTFILE "#define USERDATA NULL\n";
  print INTFILE "int main ()";
  print INTFILE " {\n";
  print INTFILE " $decs;\n";
  print INTFILE " start = clock();\n";
  print INTFILE " $integratorcall";
  print INTFILE " end = clock();\n";
  print INTFILE " cpu_time_used = ((double) (end - start)) / CLOCKS_PER_SEC;\n";
  print INTFILE " ESTIM_RE=integral[0];\n";
  print INTFILE " SIGMA_RE=error[0];\n";
  print INTFILE " CHI_RE=prob[0];\n";
  print INTFILE " ESTIM_IM=integral[1]-1.;\n";
  print INTFILE " SIGMA_IM=error[1];\n";
  print INTFILE " CHI_IM=prob[1];\n";
  if ($routine>1){print INTFILE "cout<<\"nregions = \"<<NREGIONS<<\"\\n\";\n"};
  print INTFILE " cout<<\"neval = \"<<NEVAL;\n";
  print INTFILE " cout<<\"\\nfail = \"<<FAIL;\n";
  print INTFILE " cout<<\"\\nchi squared probability:\";\n";
  print INTFILE " cout<<\"\\nreal = \"<<CHI_RE;\n";
  print INTFILE " cout<<\"\\nimag = \"<<CHI_IM;\n";
  print INTFILE " ofstream results;\n";
  print INTFILE " results.open(\"${kk}x${point}$jj\.out\");\n";
  print INTFILE " if (results.is_open())\n";
  print INTFILE " {\n";
  print INTFILE "  results << \"Real part:\\nresult = \" <<ESTIM_RE << \"\\nerror = \"\\\n";
  print INTFILE "  <<SIGMA_RE << \"\\nImaginary part:\\nresult = \" <<ESTIM_IM\\\n";
  print INTFILE "  << \"\\nerror = \" << SIGMA_IM<<\"\\nCPUtime (s) = \"<<cpu_time_used;\n";
  print INTFILE " }\n";
  print INTFILE " return 0;\n";
  print INTFILE "}\n";
  print INTFILE "int Integrand (const int *ndim, const double xx[], const int *ncomp, double ff[], void *userdata)\n";
  print INTFILE "{\n";
  print INTFILE " dcmplx f0[$numberoffunctions];\n";
  print INTFILE "$functioncall";
  print INTFILE " int ii =0;\n";
  print INTFILE " ff[0]=0.0;ff[1]=1.0;\n";
  print INTFILE " while (ii<$numberoffunctions){\n";
  print INTFILE "  ff[0]+=real(f0[ii]);ff[1]+=imag(f0[ii]);ii++;\n";
  print INTFILE " }\n";
  print INTFILE " return 0;\n";
  print INTFILE "}";
 close INTFILE;
};
sub dtoe {
 foreach $el (@_){
  $el=~s/[d|D]/e/g
 }
};

1;  
  
  
  
  



